output redirection
[c11tester.git] / main.cc
diff --git a/main.cc b/main.cc
index 215a2197d35e91fa29cf65b869a292165be4a9ad..f79366fb2d8450bcbbe69373ac45db600aec8caa 100644 (file)
--- a/main.cc
+++ b/main.cc
@@ -4,9 +4,10 @@
 
 #include <unistd.h>
 
-#include "libthreads.h"
+#include <threads.h>
 #include "common.h"
-#include "threads.h"
+#include "threads-model.h"
+#include "output.h"
 
 #include "datarace.h"
 
 
 static void param_defaults(struct model_params * params) {
        params->maxreads = 0;
+       params->maxfuturedelay = 100;
+       params->fairwindow = 0;
+       params->enabledcount = 1;
+       params->bound = 0;
+       params->maxfuturevalues = 0;
+       params->expireslop = 10;
 }
 
-static void print_usage() {
-       printf(
+static void print_usage(struct model_params *params) {
+       /* Reset defaults before printing */
+       param_defaults(params);
+
+       model_print(
 "Usage: <program name> [MC_OPTIONS] -- [PROGRAM ARGUMENTS]\n"
 "\n"
 "Options:\n"
 "-h                    Display this help message and exit\n"
-"-m                    Maximum times a thread can read from the same write while other writes exist\n"
-"--                    Program arguments follow.\n\n");
+"-m                    Maximum times a thread can read from the same write\n"
+"                      while other writes exist. Default: %d\n"
+"-M                    Maximum number of future values that can be sent to\n"
+"                      the same read. Default: %d\n"
+"-s                    Maximum actions that the model checker will wait for\n"
+"                      a write from the future past the expected number of\n"
+"                      actions. Default: %d\n"
+"-S                    Future value expiration sloppiness. Default: %u\n"
+"-f                    Specify a fairness window in which actions that are\n"
+"                      enabled sufficiently many times should receive\n"
+"                      priority for execution. Default: %d\n"
+"-e                    Enabled count. Default: %d\n"
+"-b                    Upper length bound. Default: %d\n"
+"--                    Program arguments follow.\n\n",
+params->maxreads, params->maxfuturevalues, params->maxfuturedelay, params->expireslop, params->fairwindow, params->enabledcount, params->bound);
        exit(EXIT_SUCCESS);
 }
 
 static void parse_options(struct model_params *params, int *argc, char ***argv) {
-       const char *shortopts = "hm:";
+       const char *shortopts = "hm:M:s:S:f:e:b:";
        int opt;
        bool error = false;
        while (!error && (opt = getopt(*argc, *argv, shortopts)) != -1) {
                switch (opt) {
                case 'h':
-                       print_usage();
+                       print_usage(params);
+                       break;
+               case 's':
+                       params->maxfuturedelay = atoi(optarg);
+                       break;
+               case 'S':
+                       params->expireslop = atoi(optarg);
+                       break;
+               case 'f':
+                       params->fairwindow = atoi(optarg);
+                       break;
+               case 'e':
+                       params->enabledcount = atoi(optarg);
+                       break;
+               case 'b':
+                       params->bound = atoi(optarg);
                        break;
                case 'm':
                        params->maxreads = atoi(optarg);
                        break;
+               case 'M':
+                       params->maxfuturevalues = atoi(optarg);
+                       break;
                default: /* '?' */
                        error = true;
                        break;
                }
        }
-       (*argc) -= optind;
-       (*argv) += optind;
+       (*argv)[optind - 1] = (*argv)[0];
+       (*argc) -= (optind - 1);
+       (*argv) += (optind - 1);
+       optind = 1;
 
        if (error)
-               print_usage();
+               print_usage(params);
 }
 
 int main_argc;
 char **main_argv;
 
-/** The real_main function contains the main model checking loop. */
-static void real_main() {
+/** Wrapper to run the user's main function, with appropriate arguments */
+void wrapper_user_main(void *)
+{
+       user_main(main_argc, main_argv);
+}
+
+/** The model_main function contains the main model checking loop. */
+static void model_main() {
        thrd_t user_thread;
        struct model_params params;
 
@@ -76,12 +125,14 @@ static void real_main() {
        snapshotObject->snapshotStep(0);
        do {
                /* Start user program */
-               model->add_thread(new Thread(&user_thread, (void (*)(void *)) &user_main, NULL));
+               model->add_thread(new Thread(&user_thread, &wrapper_user_main, NULL));
 
                /* Wait for all threads to complete */
                model->finish_execution();
        } while (model->next_execution());
 
+       model->print_stats();
+
        delete model;
 
        DEBUG("Exiting\n");
@@ -89,12 +140,15 @@ static void real_main() {
 
 /**
  * Main function.  Just initializes snapshotting library and the
* snapshotting library calls the real_main function.
  */
 int main(int argc, char ** argv) {
        main_argc = argc;
        main_argv = argv;
 
+       /* Configure output redirection for the model-checker */
+       redirect_output();
+
        /* Let's jump in quickly and start running stuff */
-       initSnapShotLibrary(10000, 1024, 1024, 1000, &real_main);
+       initSnapshotLibrary(10000, 1024, 1024, 4000, &model_main);
 }
@@ -89,12 +140,15 @@ static void real_main() {
 /**
  * Main function.  Just initializes snapshotting library and the
- * snapshotting library calls the real_main function.
+ * snapshotting library calls the model_main function.
  */
 int main(int argc, char ** argv) {